using System;
using System.IO;
using System.Collections;
using User;
using Task;
using Find;
using Udf;

namespace example {
	public class TaskProperties {
	    int STRING = 0;
        int FLOAT = 1;
        int DATE = 2;
        int LIST = 3;
        int MULTILIST = 6;
        int TASK = 7;
        int USER = 8;
        int INTEGER = 4;
        int MEMO = 5;
        int URL = 9;

	    UserService uSrv = new UserService();
		TaskService tSrv = new TaskService();
		UdfService udfSrv = new UdfService();
		FindService fSrv = new FindService();
		private string url = "";
		private string login = "";
		private string password = "";
		private string taskNumber = "";

		public TaskProperties() {
		    SetUp();
            string sessionId = uSrv.authenticate(login, password);
            Task.TaskBean tb = tSrv.findTaskByNumber(sessionId, taskNumber);
            if(tb == null) {
                Console.WriteLine("Task not found");
                return;
            }
            Console.WriteLine("Task Id: " + tb.id);
            Console.WriteLine("Task Name: " + tb.name);
            Console.WriteLine("Task Short Name: " + tb.shortname);
            Console.WriteLine("Task Number: " + tb.number);
            if(tb.budget != 0)
                Console.WriteLine("Task Budget: " + tb.budget);
            if(tb.abudget != 0)
                Console.WriteLine("Task Actual Budget: " + tb.abudget);
            if(tb.submitdate != -1)
                Console.WriteLine("Task Submit Date: " + getDateTime(tb.submitdate));
            if(tb.updatedate != -1)
                Console.WriteLine("Task Update Date: " + getDateTime(tb.updatedate));
            if(tb.deadline != -1)
                Console.WriteLine("Task Deadline: " + getDateTime(tb.deadline));
            if(tb.closedate != -1)
                Console.WriteLine("Task Close Date: " + getDateTime(tb.closedate));
            if(tb.categoryId != null)
                Console.WriteLine("Task Category: " + fSrv.findCategoryById(sessionId, tb.categoryId).name);
            if(tb.statusId != null)
                Console.WriteLine("Task Status: " + fSrv.findStatusById(sessionId, tb.statusId).name);
            if(tb.resolutionId != null)
                Console.WriteLine("Task Resolution: " + fSrv.findResolutionById(sessionId, tb.resolutionId).name);
            if(tb.priorityId != null)
                Console.WriteLine("Task Priority: " + fSrv.findPriorityById(sessionId, tb.priorityId).name);
            if(tb.submitterId != null)
                Console.WriteLine("Task Submitter: " + fSrv.findUserById(sessionId, tb.submitterId).name);
            if(tb.handlerUserId!= null)
                Console.WriteLine("Task Handler: " + fSrv.findUserById(sessionId, tb.handlerUserId).name);

            Udf.UdfValueBean[] udfValues = udfSrv.getUdfValues(sessionId, tb.id);

            Hashtable udfValuesMap = new Hashtable();
            for (int i = 0; i < udfValues.Length; i++) {
                if (udfValues[i] != null && udfValues[i].udfId != null) {
                    Udf.UdfValueBean uvb = udfValues[i];
                    Find.UdfBean ub = fSrv.findUDFById(sessionId, uvb.udfId);
                    int currentUdfType = ub.type;
                    string udfValue = null;
                    string[] udflistIds = uvb.udflistId;
                    if(uvb != null && !uvb.empty) {
                        if (currentUdfType == STRING || currentUdfType == MEMO || currentUdfType == URL)
                            udfValue = uvb.str;
                        if (currentUdfType == DATE)
                            udfValue = getDateTime(uvb.dat).ToString();
                        if (currentUdfType == LIST)
                            udfValue = uvb.udflistId != null ? uvb.udflistId[0] : null;
                        if (currentUdfType == FLOAT)
                            udfValue = uvb.num + "";
                        if (currentUdfType == INTEGER)
                            udfValue = uvb.num + "";
                        if (currentUdfType == URL) {
                            string val;
                            val = udfValue;
                            string url = "";
                            string description = "";
                            if (val != null) {
                                int j = val.IndexOf("\n");
                                if (j > 1 && j < val.Length - 1) {
                                    url = val.Substring(0, j);
                                    description = val.Substring(j);
                                } else if (j == 0) {
                                    url = "";
                                    description = val;
                                } else
                                    url = val;
                            }
                            udfValue = url;
                        }
                        if (currentUdfType == DATE)
                            udfValue = udfValue;
                        if (currentUdfType == MEMO)
                            udfValue = udfValue == null ? null : udfValue.ToString();
                        if (currentUdfType == LIST && udfValue != null) {
                            Udf.UdflistBean[] udflistArray = udfSrv.getUdflist(sessionId, ub.udfId);
                            for (int j = 0; j < udflistArray.Length; j++)
                                if (udflistArray[j].id.Equals(udfValue))
                                    udfValue = udflistArray[j].val;
                        }
                        if (currentUdfType == MULTILIST) {
                            Udf.UdflistBean[] udflistArray = udfSrv.getUdflist(sessionId, ub.udfId);
                            for (int j = 0; j < udflistArray.Length; j++) {
                                for (int k = 0; k < udflistIds.Length; k++)
                                    if (udflistArray[j].id.Equals(udflistIds[k]))
                                        udfValue += udflistArray[j].val + "|";
                            }
                        }
                        if (currentUdfType == TASK) {
                            string[] sl = uvb.taskListId;
                            for (int j = 0; j < sl.Length; j++)
                                udfValue += "#"+fSrv.findTaskById(sessionId, sl[j]).number + "|";
                        }
                        if (currentUdfType == USER) {
                            string[] sl = uvb.userListId;
                            for (int j = 0; j < sl.Length; j++)
                                udfValue += fSrv.findUserById(sessionId, sl[j]).name +"|";
                        }
                        Console.WriteLine(ub.caption + ": " + udfValue);

                    }
                }
            }

        }

	    public void SetUp() {
	        StreamReader sr = new StreamReader(new FileStream("account.properties", FileMode.Open, FileAccess.Read));
            string s = null;
            while((s = sr.ReadLine()) != null) {
	            int i = s.IndexOf("ts.url");
	            if(i != -1) {
    	            url = s.Substring(i + "ts.url".Length).Trim();
	                if(url[0] == '=' && url.Length > 1)
	                    url = url.Substring(1).Trim();
	            }
    	        i = s.IndexOf("ts.login");
	            if(i != -1) {
	                login = s.Substring(i + "ts.login".Length).Trim();
	                if(login[0] == '=' && login.Length > 1)
	                    login = login.Substring(1).Trim();
    	        }
	            i = s.IndexOf("ts.password");
	            if(i != -1) {
	                password = s.Substring(i + "ts.password".Length).Trim();
	                if(password[0] == '=' && password.Length > 1)
    	                password = password.Substring(1).Trim();
        	    }
        	    i = s.IndexOf("ts.taskNumber");
				if(i != -1)
				{
					taskNumber = s.Substring(i + "ts.taskNumber".Length).Trim();
					if(taskNumber[0] == '=' && taskNumber.Length > 1)
						taskNumber = taskNumber.Substring(1).Trim();
				}
	        }
	        uSrv.Url = url + "/services/User";
        	tSrv.Url = url + "/services/Task";
    	    fSrv.Url = url + "/services/Find";
    	    udfSrv.Url = url + "/services/Udf";
	    }

    	public static DateTime getDateTime(long time) {
           long win32FileTime = 10000*(long)time + 116444736000000000;
           return DateTime.FromFileTimeUtc(win32FileTime);
        }
        public static void Main(string[] args) {
    		new TaskProperties();
    	}
    }
}